import time
from multiprocessing import RLock
from pyconnectedcars.connection import Connection
from pyconnectedcars.Fuel import Fuel
from pyconnectedcars.BinarySensor import SystemsAreOkSensor, OilLevelIsOkSensor, TirePressureIsOkSensor, BatteryChargeIsOkSensor, LockSensor
from pyconnectedcars.GPS import GPS, Odometer


class Controller:
    def __init__(self, email, password, update_interval, baseurl=""):
        self.__connection = Connection(email, password, baseurl)
        self.__vehicles = []
        self.update_interval = update_interval
        self.__car = {}
        self.__last_update_time = 0
        self.__lock = RLock()
        self.update()
        for car_id, car in self.__car.items():
            self.__vehicles.append(Fuel(car, self))
            self.__vehicles.append(GPS(car, self))
            self.__vehicles.append(Odometer(car, self))
            self.__vehicles.append(SystemsAreOkSensor(car, self))
            self.__vehicles.append(OilLevelIsOkSensor(car, self))
            self.__vehicles.append(TirePressureIsOkSensor(car, self))
            self.__vehicles.append(BatteryChargeIsOkSensor(car, self))
            self.__vehicles.append(LockSensor(car, self))

    def get(self):
        self.__last_update_time = time.time()
        return self.__connection.get_data()

    def list_vehicles(self):
        return self.__vehicles

    def update(self):
        cur_time = time.time()
        with self.__lock:
            if cur_time - self.__last_update_time > self.update_interval:
                data = self.get()
                if data and data['data']:
                    for car_data in data['data']['user']['cars']:
                        self.__car[car_data['id']] = car_data
                else:
                    self.__car = {}

    def get_car_params(self, car_id):
        return self.__car[car_id]



from pyconnectedcars.vehicle import VehicleDevice
import datetime


class Fuel(VehicleDevice):
    def __init__(self, data, controller):
        super().__init__(data, controller)
        self.__fuel_level = 0
        self.fuel_level_pct = 0

        self.last_updated = 0

        self.type = 'fuel level'
        self.measurement = 'VOLUME_LITERS'
        self.hass_type = 'sensor'
        self.name = self._name()
        self.uniq_name = self._uniq_name()
        self.bin_type = 0x5
        self.update()

    def update(self):
        self._controller.update()
        data = self._controller.get_car_params(self._id)
        if data:
            self.__fuel_level = data['fuelLevelLiter']
            self.fuel_level_pct = data['fuelLevel']
            self.last_updated = datetime.datetime.strptime(data['fuelLevelUpdatedAt'], "%Y-%m-%dT%H:%M:%S.%fZ")

    @staticmethod
    def has_battery():
        return False

    def get_value(self):
        return self.__fuel_level



from setuptools import setup
setup(
    name='pyconnectedcars',
    version='0.2.1',
    packages=['pyconnectedcars'],
    include_package_data=True,
    python_requires='>=3',
    license='MIT',
    description='A library to work with Connected Cars API.',
    long_description='A library to work with Connected Cars car API.',
    url='https://github.com/DarkFox/pyconnectedcars',
    author='Martin Eberhardt',
    author_email='martin@darkfox.dk',
    classifiers=[
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Internet',
    ],
)
